import std/[os, strutils]
import ./config, ./library, ./libvlc, ./models
import private/cliconstants

type CliCtx = ref object
  cfg: WdiwtltConfig
  library: WdiwtltLibrary
  vlc: LibVlcInstance
  player: VlcMediaPlayer

const STOP_CMDS = ["stop", "exit", "quit", ":q"]
var cmdChannel: Channel[string]
var readyForCmd: Channel[bool]

proc handleCmd(ctx: CliCtx, cmd: string): bool;

proc initCliCtx(cfg: WdiwtltConfig): CliCtx =
  let vlc = newVlc()
  return CliCtx(
    cfg: cfg,
    player: vlc.newMediaPlayer,
    vlc: vlc,
    library: initLibrary(cfg.libraryPath, cfg.dbPath))

proc release(ctx: CliCtx) =
  if not ctx.player.isNil: ctx.player.release
  if not ctx.vlc.isNil: ctx.vlc.release

proc readInputLoop() =
  var line: string
  while not STOP_CMDS.contains(line):
    stdout.write("> ")
    line = stdin.readline
    cmdChannel.send(line)

    while not readyForCmd.tryRecv().dataAvailable: sleep(200)


proc renderLoop(ctx: CliCtx) =
  var shouldStop = false
  while not shouldStop:
    # poll for input
    let cmdMsg = cmdChannel.tryRecv()
    if cmdMsg.dataAvailable:
      shouldStop = ctx.handleCmd(cmdMsg.msg)
      readyForCmd.send(true)

    # render
    sleep(200)

proc startCli*(cfg: WdiwtltConfig) =
  var ctx = cfg.initCliCtx

  try:
    stdout.writeLine("wdiwtlt v" & WDIWTLT_VERSION)
    cmdChannel.open()
    readyForCmd.open()

    var readLineWorker: Thread[void]
    createThread(readLineWorker, readInputLoop)
    ctx.renderLoop()
    readLineWorker.joinThread()

  finally:
    readyForCmd.close()
    cmdChannel.close()
    ctx.release()

proc processScan(ctx: CliCtx) =
  ctx.library.scan

proc handleCmd(ctx: CliCtx, cmd: string): bool =
  result = false
  if STOP_CMDS.contains(cmd): return true

  let cmdParts = cmd.split(" ")
  if cmdParts.len == 0: return

  case cmdParts[0]:
    of "scan": ctx.processScan()
    else: stdout.writeLine("Unrecognized command: '" & cmdParts[0] & "'")
